#!/usr/bin/perl
use lib '/home/erealms/ethereal/mgmt/perl';

#################################################################################
# Created       : Martin Foster
# Modified      : 30-Oct-2006
#################################################################################
#
# Realm - Script part of Ethereal Realms designed to interact within realms
# Copyright (C) 2000-2006  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

use CGI;							# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Comm;						# Communication handler
use Ethereal::Database;						# Database handler
use Ethereal::Login;						# Login functionality
use Ethereal::Param;						# Parameter control
use Ethereal::Post;						# Post handler
use Ethereal::Pull;						# Post extracts
use Ethereal::Template;						# Template management
use Ethereal::Who;						# Who is on

#################################################################################
# Gobal override
#################################################################################
$CGI::POST_MAX=1024 * 50;					# Maximum posts
$CGI::DISABLE_UPLOADS = 1;					# Disable uploads

#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $comm;							# Communications panel handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle
my $pull;							# Pull handle
my $post;							# Post handle
my $tmpl;							# Template management

my $chatpref;							# Chat preferences
my $icomm;							# Comm panel inline parameters
my $imain;							# Main panel inline parameters
my $javascript;							# Javascript preferences
my $stm;							# Stream URL
my $url;							# Basic URL

my $frame;							# Columns

my %args;							# Arguments
my %realm;							# Realm hash

#################################################################################
# Program Area
#################################################################################

	# Initial handle
	$cgi      = new CGI;	
	$database = new Ethereal::Database();

	# Connect and fetch
	$database->Connect($cgi);


	# Parameter handling
	$param    = new Ethereal::Param($database, $cgi);
	$param->GetParam();

	# Fetch hash
	$database->GetHashRealm(\%realm);

	# Fetch URL
	$url = $cgi->url(-absolute=>1);

	# Stream URL
	$stm = $url;
	$stm =~ s/^(.+)\/.+$/$1\/stream.pl/gs;

	
	# Login handle
	$login = new Ethereal::Login($database, $cgi, $param);

	# Disable header generation
	$login->{'AUTOMATE'} = 0;


	# Authenthication
	if ($login->GetVerificationRealm())
	{
		# Make sure logins are possible
		$login->CheckLegality(\%realm);

		# Realm specific handles
		$tmpl  = new Ethereal::Template(\%realm);
		$comm  = new Ethereal::Comm($database, $cgi, $tmpl);
		$post  = new Ethereal::Post();
		$pull  = new Ethereal::Pull();


		# Verify for need for extract
		$pull->Posts($database, $cgi, $param);
	

		# Retreive javascript and chat preferences
		($javascript, $chatpref) = $database->DataGetChatOptions($param->{'USER'});

		# Handle post writing
		$comm->DataWrite($param, $chatpref);


		# Basic arguments
		%args = (
			USER   => $param->{'USER'},
			CRYPT  => $param->{'CRYPT'},
			CHAR   => $param->{'CHAR'},
			ROOM   => $param->{'ROOM'},
		 );

		# Add in password
		$args{'RCRYPT'} = $param->{'RCRYPT'}
		  if (defined($param->{'RCRYPT'}));
		
		# Only if necessary
		$args{'PSTART'} = $param->{'PSTART'}
		  if (($chatpref eq 'stream') or ($chatpref eq 'autoscroll'));

		# Framed chat
		if ($chatpref eq 'framed')
		{
			# Only do following if called with no parameters
			unless ((defined($param->{'MAIN'})) 
			     || (defined($param->{'COMM'})))
			{
				# Retreive parameter list
				$icomm = $param->EmbedInline($param->Flat());
				$imain = $param->EmbedInline(%args);

				# Retreive frameset
				($frame) = $database->FetchFrameNorm($param->{'ROOM'});

				# Rewrite header
				$frame =~ s/MNAME/$param->{'ROOM'}/gs;
				$frame =~ s/LMAIN/$url$imain&MAIN=True/gs;
				$frame =~ s/LCOMM/$url$icomm&COMM=True/gs;


				# Print file header
				$login->WriteHeader();

				# Display frameset
				print $frame, "\n";
			}
	
			# Message display area
			if (defined($param->{'MAIN'}))
			{ 
				# File and HTML header
				$login->WriteHeader(undef, -target=>'Main');
				print $database->FetchRealmHeader($param->{'ROOM'});

				# Display banner
				$database->ShowBanner($param);


				# Posting related
				$post->HandlePosts($database, $cgi,  $param, 'TRUE'); 

				# Parameter handler
				$param->Cleanup('MAIN');


				# Activation button
				print $cgi->start_form();
				print $param->EmbedNormal($param->Flat(), MAIN=>'True'), "\n";
				

				# Display refresh
				$post->RefreshFramed($database, $cgi, $param);


				print $cgi->end_form(), "\n";
				print $database->FetchRealmFooter($param->{'ROOM'});
			}

			# Comm panel
			if (defined($param->{'COMM'}))
			{ 
				# Communications panel
				$comm->CommIndependant($login, $param); 
			}
		}


		# Streaming chat with autoscroll
		elsif (($chatpref eq 'autoscroll') && ($javascript eq 'yes')) 
		{
			# Only do following if called with no parameters
			unless ((defined($param->{'MAIN'}))
			 || (defined($param->{'COMM'}))
			 || (defined($param->{'SCRL'})))
			{
				# Retreive parameter list
				$icomm = $param->EmbedInline($param->Flat());
				$imain = $param->EmbedInline(%args);

				# Retreive frameset
				($frame) = $database->FetchFrameAuto($param->{'ROOM'});

				# Rewrite header
				$frame =~ s/MNAME/$param->{'ROOM'}/gs;
				$frame =~ s/LSCRL/$url$imain&SCRL=True/gs;
				$frame =~ s/LMAIN/$stm$imain&MAIN=True/gs;
				$frame =~ s/LCOMM/$url$icomm&COMM=True/gs;


				# Print file header
				$login->WriteHeader();

				# Display frameset
				print $frame, "\n"
			}	
	
			# Message display area
			if (defined($param->{'MAIN'}))
			{ 
				# Legacy override
				$param->{'OVER'} = 'true';

				
				# Flush standard output				
				$| = 1;
				print "";

				# File header
				$login->WriteHeader(undef, -target=>'Main');
				print $database->FetchRealmHeader($param->{'ROOM'});

				# Display banner
				$database->ShowBanner($param);


				# Java enabled post window
				$post->HandleStream($database, $cgi, $param); 

				# Document footer
				print $database->FetchRealmFooter($param->{'ROOM'});
			}

			# Comm panel
			if (defined($param->{'COMM'}))
			{
				$comm->CommAutoScroll($login, $param);
			}

			# Remote control
			if (defined($param->{'SCRL'}))
			{
				$login->WriteHeader(undef, -target=>'Remote');
				print $database->FetchRealmRemote($param->{'ROOM'});
			}
		}

		# Streaming chat
		elsif (($chatpref eq 'stream') || ($chatpref eq 'autoscroll'))
		{
			# Only do following if called with no parameters
			unless ((defined($param->{'MAIN'}))
			     || (defined($param->{'COMM'})))
			{
				# Retreive parameter list
				$icomm = $param->EmbedInline($param->Flat());
				$imain = $param->EmbedInline(%args);

				# Retreive frameset
				($frame) = $database->FetchFrameNorm($param->{'ROOM'});

				# Rewrite header
				$frame =~ s/MNAME/$param->{'ROOM'}/gs;
				$frame =~ s/LMAIN/$stm$imain&MAIN=True/gs;
				$frame =~ s/LCOMM/$url$icomm&COMM=True/gs;


				# Print file header
				$login->WriteHeader();

				# Display frameset
				print $frame, "\n";
			}	
	
			# Message display area
			if (defined($param->{'MAIN'}))
			{ 
				# Legacy override
				$param->{'OVER'} = 'true';

				
				# Flush standard output				
				$| = 1;
				print "";

				# File header
				$login->WriteHeader(undef, -target=>'Main');
				print $database->FetchRealmHeader($param->{'ROOM'});

				# Display banner
				$database->ShowBanner($param);


				# Posting related
				$post->HandleStream($database, $cgi, $param); 

				# Document footer
				print $database->FetchRealmFooter($param->{'ROOM'});
			}

			# Comm panel
			if (defined($param->{'COMM'}))
			{ 
				$comm->CommStream($login, $param); 
			}
		}


		# Basic chat system
		else
		{
			# Print file and HTML header
			$login->WriteHeader();
			print $database->FetchRealmHeader($param->{'ROOM'});

			# Display banner
			$database->ShowBanner($param);


			# Message display and posting
			$post->HandlePosts($database, $cgi, $param); 

			# Spacer
			print "<BR><BR>\n";

			$comm->CommDependant($login, $param);

			# Document footer
			print $database->FetchRealmFooter($param->{'ROOM'});
		}
	}


#################################################################################
# Sub-Routines
#################################################################################
